#ifndef _KYSEC_NETCTL_H
#define _KYSEC_NETCTL_H

#include <sys/types.h>
#include <limits.h>

#define KYSEC_XML_PATH	"/etc/kysec/netctl/netctl.xml"
#define KYSEC_PKG_XML_PATH	"/etc/kysec/netctl/netctl_pkg.xml"
#define XML_HASH_SIZE	64

#define KYSEC_NOTIFY_MESG_MAX 1000

//#define PKG_NAME_MAX 25
#define PKG_NAME_MAX 4096

enum network_op_type {
	OP_UNKNOWN = 0,
	OP_ALLOW = 1,
	OP_DENY,
	OP_AUTH
};

enum netctl_server_type {
	SERV_UNKNOWN = 0,
	SERV_SYSTEM,
	SERV_USER,
	SERV_NETWORK
};

struct kysec_netctl {
	char path[PATH_MAX];
	char hash[XML_HASH_SIZE + 1];
	int type;
	char name[NAME_MAX + 1];
	char ico[PATH_MAX];
	int server_type;
};

struct kysec_netctl_ext {
	char path[PATH_MAX];
	char hash[XML_HASH_SIZE + 1];
	int type;
	char name[NAME_MAX + 1];
	char ico[PATH_MAX];
	char desktop[PATH_MAX];
	int server_type;
};

struct kysec_netctl_pkg {
	char pkg[PKG_NAME_MAX];
	int type;
	char ico[PATH_MAX];
	char desktop[PATH_MAX];
	int server_type;
};

struct kysec_netctl_pkg_path {
	char pkg[PKG_NAME_MAX];
	char path[PATH_MAX];
	int type;
	char ico[PATH_MAX];
	char desktop[PATH_MAX];
	int server_type;
};

struct kysec_netctl_kernel {
	char path[PATH_MAX];
	char hash[XML_HASH_SIZE + 1];
	int type;
};

#ifdef __cplusplus
extern "C" {
#endif

/* netctl.xml文件存放的是应用网络管控的信息 */

/**
 * 获取文件路径+uid的hash值
 */
char *kysec_netctl_get_path_hash(const char *filepath, uid_t uid);

/**
 * 获取用户区分功能标识值
 * 返回值：0:不区分用户，1:区分用户，-1:出现错误
 */
int kysec_netctl_get_user_distinguish(void);

/**
 * 设置用户区分功能标识值
 * @value: 表示是否进行用户区分，0表示不区分，1表示区分，其他值为非法值
 * 返回值：0:成功，非0:失败
 */
int kysec_netctl_set_user_distinguish(int value);

/**
 * 添加一个应用信息节点到xml文件
 * @path: 程序路径
 * @type: 操作类型，enum network_op_type
 * @name: 服务名字
 * @ico: 服务的图标路径
 * @server_type: 服务类型，enum netctl_server_type
 * 返回值: 0:成功，1:包含该@path的节点已经存在，-1:出错
 */
int kysec_netctl_add(uid_t uid, char *path, int type,
		char *name, char *ico, int server_type);

/**
 * 添加一个应用信息节点到xml文件
 * @path: 程序路径
 * @type: 操作类型，enum network_op_type
 * @name: 服务名字
 * @ico: 服务的图标路径
 * @desktop: desktop文件路径
 * @server_type: 服务类型，enum netctl_server_type
 * 返回值: 0:成功，1:包含该@path的节点已经存在，-1:出错
 */
int kysec_netctl_ext_add(uid_t uid, char *path, int type,
		char *name, char *ico, char *desktop, int server_type);

/* 从xml文件中删除一个节点
 * @path: 程序路径
 * 返回值: 0:成功，-1:出错
 */
int kysec_netctl_remove(uid_t uid, char *path);

/* 更新xml文件的某一个节点
 * @path: 程序路径
 * @type: 操作类型，enum network_op_type，
 *        若type != OP_UNKNOWN则更新节点里的type值
 * @name: 服务名字，
 *        若name != NULL则更新节点里的name值
 * @ico: 服务的图标路径，
 *       若ico != NULL则更新节点里的ico值
 * @server_type: 服务类型，enum netctl_server_type，
 *       若server_type != SERV_UNKNOWN则更新节点中的server_type值
 *
 * 返回值: 0:成功，-1:失败
 */
int kysec_netctl_update(uid_t uid, char *path, int type,
		char *name, char *ico, int server_type);

/* 更新xml文件的某一个节点
 * @path: 程序路径
 * @type: 操作类型，enum network_op_type，
 *        若type != OP_UNKNOWN则更新节点里的type值
 * @name: 服务名字，
 *        若name != NULL则更新节点里的name值
 * @ico: 服务的图标路径，
 *       若ico != NULL则更新节点里的ico值
 * @desktop: 服desktop路径，
 *       若desktop != NULL则更新节点里的desktop值
 * @server_type: 服务类型，enum netctl_server_type，
 *       若server_type != SERV_UNKNOWN则更新节点中的server_type值
 *
 * 返回值: 0:成功，-1:失败
 */
int kysec_netctl_ext_update(uid_t uid, char *path, int type,
		char *name, char *ico, char *desktop, int server_type);

/* 获取xml文件中所有节点数据
 * 返回值: kysec_xml_kernel指针数组，大小为@count
 */
struct kysec_netctl_kernel *kysec_netctl_read_all_data(int *count);
/* 获取指定用户uid在程序xml文件中的所有节点数据 */
struct kysec_netctl *kysec_netctl_read_all_data_with_uid(uid_t uid, int *count);
/* 获取指定用户指定程序对应的信息 */
struct kysec_netctl *kysec_netctl_read_node_with_uid(uid_t uid, char *filepath);

/* 获取指定用户uid在程序xml文件中的所有节点数据 */
struct kysec_netctl_ext *kysec_netctl_ext_read_all_data_with_uid(uid_t uid, int *count);

/* 获取指定用户指定程序对应的信息 */
struct kysec_netctl_ext *kysec_netctl_ext_read_node_with_uid(uid_t uid, char *filepath);


/* netctl_pkg.xml文件存放的是包网络管控的信息 */

/**
 * 添加一个包信息节点到xml文件
 * @pkg: 包名
 * @type: 操作类型，enum network_op_type
 * @ico: 包的图标路径
 * @desktop: desktop文件路径
 * @server_type: 服务类型，enum netctl_server_type
 * 返回值: 0:成功，1:包含该@path的节点已经存在，-1:出错
 */
int kysec_netctl_pkg_add(uid_t uid, char *pkg, int type,
		char *ico, char *desktop, int server_type);
int kysec_netctl_pkg_path_add(uid_t uid, char *pkg, char *path, int type,
		char *ico, char *desktop, int server_type);

/* 从xml文件中删除一个节点
 * @pkg: 包名
 * 返回值: 0:成功，-1:出错
 */
int kysec_netctl_pkg_remove(uid_t uid, char *pkg);

/* 更新xml文件的某一个节点
 * @pkg: 包名
 * @type: 操作类型，enum network_op_type，
 *        若type != OP_UNKNOWN则更新节点里的type值
 * @ico: 服务的图标路径，
 *       若ico != NULL则更新节点里的ico值
 * @desktop: 服desktop路径，
 *       若desktop != NULL则更新节点里的desktop值
 * @server_type: 服务类型，enum netctl_server_type，
 *       若server_type != SERV_UNKNOWN则更新节点中的server_type值
 *
 * 返回值: 0:成功，-1:失败
 */
int kysec_netctl_pkg_update(uid_t uid, char *pkg, int type,
		char *ico, char *desktop, int server_type);
int kysec_netctl_pkg_path_update(uid_t uid, char *pkg, char *path, int type,
		char *ico, char *desktop, int server_type);

/* 检查软件包是否安装
 * @pkgname: 包名
 *
 * 返回值: 0:未安装，1:已安装，-1:失败
 */
int check_pkg_is_install(char *pkgname);

/* 检查文件是否存在
 * @path: 文件路径
 *
 * 返回值: 0:不存在，1:存在
 */
int check_program_is_exist(char *path);

/* 获取指定用户uid在包xml文件中的所有节点数据 */
struct kysec_netctl_pkg *kysec_netctl_pkg_read_all_data_with_uid(uid_t uid, int *count);
/* 获取指定用户指定包对应的信息 */
struct kysec_netctl_pkg *kysec_netctl_pkg_read_node_with_uid(uid_t uid, char *pkg);
struct kysec_netctl_pkg_path *kysec_netctl_pkg_path_read_node_with_uid(uid_t uid, char *pkg);

#ifdef __cplusplus
}
#endif

#endif // _KYSEC_NETCTL_H
